import io
import os
from optparse import OptionParser
from os.path import basename, isfile, splitext

from clint.textui import progress
from PyPDF2 import PdfFileReader, PdfFileWriter
from reportlab.lib import colors, pagesizes
from reportlab.lib.colors import HexColor
from reportlab.lib.units import cm
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFError, TTFont
from reportlab.pdfgen import canvas


def get_paper_size(size):
    return {
        'A0': pagesizes.A0,
        'A1': pagesizes.A1,
        'A2': pagesizes.A2,
        'A3': pagesizes.A3,
        'A4': pagesizes.A4,
        'A5': pagesizes.A5,
        'A6': pagesizes.A6,
        'LETTER': pagesizes.LETTER,
        'LEGAL': pagesizes.LEGAL,
        'B0': pagesizes.B0,
        'B1': pagesizes.B1,
        'B2': pagesizes.B2,
        'B3': pagesizes.B3,
        'B4': pagesizes.B4,
        'B5': pagesizes.B5,
        'B6': pagesizes.B6,
    }.get(size, pagesizes.A4)


def get_page_size(options):
    if options.page_width and options.page_height:
        width, height = options.page_width * cm, options.page_height * cm
    else:
        width, height = get_paper_size(options.paper_size)
        if options.landscape:
            width, height = height, width
    return width, height


def test_if_file_exists(filename):
    if filename is None:
        return False
    return isfile(filename)


def get_color(color):
    return HexColor(color)


def create_font_args(options):
    if test_if_file_exists(options.font):
        font_name, font_path = (basename(splitext(options.font)[0]),
                                options.font)
        options.font_name = font_name
        options.font_path = font_path
        return False
    else:
        return True


def create_footer(options):
    if not options.text:
        return None, "No text to write, exiting"

    pdf = io.BytesIO()
    width, height = get_page_size(options)

    can = canvas.Canvas(pdf, pagesize=(width, height))
    color = options.font_color
    can.setFillColor(color)

    size = 8
    if options.font_size:
        size = options.font_size
    can.setFont(options.font_name, size)
    if options.top and options.side:
        can.drawString(
            options.side * cm, height - options.top * cm, options.text)
    else:
        can.drawCentredString(width / 2.0, height / 2.0, options.text)
    can.save()
    return pdf, None


def merge_files(options, footer):
    footer.seek(0)
    new_pdf = PdfFileReader(footer)

    if not test_if_file_exists(options.input):
        print("Could not read the input file - did you specify one?")
        exit(1)

    if not options.output:
        print("No output path specified")
        exit(1)

    try:
        book = PdfFileReader(open(options.input, "rb"))
    except (Exception, e):
        print("Unable to load input PDF - {}".format(e))
        exit(1)

    output = PdfFileWriter()

    if options.author:
        output.addMetadata({"/Author": options.author})

    if options.subject:
        output.addMetadata({"/Subject": options.subject})

    for index, page in progress.dots(enumerate(book.pages)):
        page = book.getPage(index)
        if index >= options.skip_pages:
            page.mergePage(new_pdf.getPage(0))
        output.addPage(page)

    outputStream = open(options.output, "wb")
    output.write(outputStream)
    outputStream.close()
    print("Written {}".format(options.output))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

readme = open('README.md').read()
history = open('HISTORY.md').read().replace('.. :changelog:', '')
version = "1.5.6"

setup(
    name='gerbil',
    version=version,
    description='Tool for adding text to the bottom of PDF pages.',
    long_description=readme + '\n\n' + history,
    author='Ben Hughes',
    author_email='bwghughes@gmail.com',
    url='https://github.com/bwghughes/gerbil',
    include_package_data=True,
    install_requires=[
        'Pillow==2.9.0',
        'args==0.1.0',
        'clint==0.4.1',
        'reportlab==3.2.0',
        'pypdf2==1.25.1',
    ],
    packages=[
        'gerbil',
    ],
    package_dir={'gerbil': 'gerbil'},
    license="BSD",
    zip_safe=False,
    keywords='gerbil',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.4',
    ],
    test_suite='tests',
    entry_points={
        'console_scripts':
            ['gerbil=gerbil:main']
    }

)



from optparse import OptionParser
from .gerbil import create_footer, merge_files, create_font_args, get_color

from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont

import pkg_resources  # part of setuptools
try:
    version = pkg_resources.require("gerbil")[0].version
except:
    pass


def main():
    parser = OptionParser(version=version)
    parser.add_option('-t', '--text',
                      help="The text to appear on footer the page.")
    parser.add_option('-i', '--input',
                      help="The input file for the text to be addedso.")
    parser.add_option('-o', '--output',
                      help="The ouput file to be saved.")
    parser.add_option('-f', '--font',
                      default="Bliss-Regular.ttf",
                      help="Path to the TrueType font file to be used (*.ttf)")
    parser.add_option('-c', '--font-color',
                      default=get_color("#545454"),
                      help="Hex color, defaults to Grey: #545454")
    parser.add_option('-s', '--font-size',
                      type="float",
                      default=8,
                      help="The font size px to be used (default = 8)")
    parser.add_option('-a', '--author',
                      help="The author to appear in metadata.")
    parser.add_option('-u', '--subject',
                      help="The subject to appear in metadata.")
    parser.add_option('--top',
                      type="float",
                      help="The padding from the left hand side of the page (cm)")
    parser.add_option('--side',
                      type="float",
                      help="The padding from the top of the page (cm)")
    parser.add_option('-x', '--page-width',
                      type="float",
                      help="The width of the page (cm)")
    parser.add_option('-y', '--page-height',
                      type="float",
                      help="The height of the page (cm)")
    parser.add_option('--paper-size',
                      default="A4",
                      help="Default = A4. The named size of the paper Supported: A0 - A6, B0 - B6, LETTER, LEGAL.  Paramter ignored if -x and -y are supplied ")
    parser.add_option('--landscape',
                      action="store_true", dest="landscape", default=False,
                      help="Default = portrait unless this flag is supplied. Defines the page orientation,  (ignored if -x and -y are given)")
    parser.add_option('--skip-pages',
                      type="int",
                      default=0,
                      help="number of pages to skip before stamping starts. Default = 0"
                      )
    (options, args) = parser.parse_args()

    err = create_font_args(options)

    if not err:
        pdfmetrics.registerFont(TTFont(options.font_name, options.font_path))
        f, err = create_footer(options)
        if err is None:
            merge_files(options, f)
        else:
            print(err)
    else:
        print("Can't find {} Exiting".format(options.font))
        exit(1)
